@@ -20,8 +20,8 @@
 #include <bakery/App/Dialog_OfferSave.h>
 #include <bakery/App/Dialog_OverwriteFile.h>
 #include <gconfmm.h>
+#include <libgnomevfsmm.h>
 #include <libintl.h>
-#include <fstream>
 #include <algorithm>
 
 //#include <gtk/gtkfilesel.h>
@@ -31,21 +31,13 @@
 namespace Bakery
 {
 
-
-//Initialize static member data:
-App_WithDoc::type_vecStrings App_WithDoc::m_vecDocHistory;
+App_WithDoc::type_list_strings App_WithDoc::m_mime_types;
 
 App_WithDoc::App_WithDoc(const Glib::ustring& appname)
-: App(appname)
+: App(appname),
+  m_pDocument(0),
+  m_bCloseAfterSave(false)
 {  
-  m_pDocument = 0;
-  m_bCloseAfterSave = false;
-
-  m_vecDocHistory.resize(4); //TODO: Make the size a user-defined preference.
-  
-  //If this is the first instance, then load the configuration:
-  //if(m_AppInstanceManager.get_app_count() == 0)
-    //session_load_document_history();
 }
 
 App_WithDoc::~App_WithDoc()
@@ -53,17 +45,24 @@ App_WithDoc::~App_WithDoc()
   //Delete the document:
   if(m_pDocument)
   {
-    delete m_pDocument;
+    delete m_pDocument; //This will cause Document::signal_forget to be emitted, so the Views will then null their pointers as well. A smartpointer might be a better way to do this.
     m_pDocument = 0;
   }
 }
 
+//static
+void  App_WithDoc::add_mime_type(const Glib::ustring& mime_type)
+{
+  if( std::find(m_mime_types.begin(), m_mime_types.end(), mime_type) == m_mime_types.end() )
+    m_mime_types.push_back(mime_type);
+}
+
 void App_WithDoc::on_menu_file_close()
 {
   if(m_pDocument->get_modified())
   {
     //Offer to save changes:
-    m_bCloseAfterSave = true; //Checked in FileSelection signal handler.
+    m_bCloseAfterSave = true; //Checked in FileChooser signal handler.
     offer_to_save_changes(); //If a File|Exit is in progress, this could cancel it.
   }
 
@@ -72,7 +71,7 @@ void App_WithDoc::on_menu_file_close()
 }
 
 
-bool App_WithDoc::open_document(const Glib::ustring& strFilepath)
+bool App_WithDoc::open_document(const Glib::ustring& file_uri)
 {
   //Check whether it's already open:
   //It could even be open in this instance.
@@ -86,7 +85,7 @@ bool App_WithDoc::open_document(const Glib::ustring& strFilepath)
     if(pApp)
     {
       Document* pDoc = pApp->get_document();
-      if(pDoc->get_filepath() == strFilepath)
+      if(pDoc->get_file_uri() == file_uri)
       {
         bAlreadyOpen = true;
         pAppAlreadyOpen = pApp;
@@ -127,7 +126,7 @@ bool App_WithDoc::open_document(const Glib::ustring& strFilepath)
     }
 
     //Open it.
-    pApp->m_pDocument->set_filepath(strFilepath);
+    pApp->m_pDocument->set_file_uri(file_uri);
     bool bTest = pApp->m_pDocument->load();
 
     if(!bTest) //if open failed.
@@ -158,7 +157,7 @@ bool App_WithDoc::open_document(const Glib::ustring& strFilepath)
       set_document_modified(false); //disables menu and toolbar Save items.
 
       //Update document history list:
-      document_history_add(strFilepath);
+      document_history_add(file_uri);
 
       return true; //success.
     }
@@ -175,22 +174,9 @@ void App_WithDoc::on_menu_file_open()
   ui_bring_to_front();
 
   //Ask user to choose file to open:
-  Glib::ustring strFilepath = ui_file_select_open();
-  if(!strFilepath.empty())
-    open_document(strFilepath);
-}
-
-void  App_WithDoc::on_menu_file_open_recent(guint index)
-{
-  if(index < m_vecDocHistory.size())
-  {
-    const Glib::ustring& strFilepath = m_vecDocHistory[index];
-    bool bTest = open_document(strFilepath);
-
-    //Remove it from the recent documents list if it couldn't be opened.
-    if(!bTest)
-      document_history_remove(strFilepath);
-  }
+  Glib::ustring file_uri = ui_file_select_open();
+  if(!file_uri.empty())
+    open_document(file_uri);
 }
 
 //This is for calling directly, use the other override for the signal handler:
@@ -208,29 +194,36 @@ void App_WithDoc::on_menu_file_saveas()
   ui_bring_to_front();
 
   //Show the save dialog:  
-  const Glib::ustring& strFilepathOld = m_pDocument->get_filepath();
-  Glib::ustring strFilepath = ui_file_select_save(strFilepathOld);
-  if(!strFilepath.empty())
+  const Glib::ustring& file_uriOld = m_pDocument->get_file_uri();
+  Glib::ustring file_uri = ui_file_select_save(file_uriOld);
+  if(!file_uri.empty())
   {
-    bool bUseThisFilePath = true;
+    bool bUseThisFileUri = true;
 
     //Check whether file exists already:
     {
-      std::ifstream fStream(strFilepath.c_str());
-      if(fStream.is_open())
+      // Try to open the input file.
+      Gnome::Vfs::Handle read_handle;
+      try
       {
-        //Ask user to confirm overwrite:
-        bool bOverwrite = ui_ask_overwrite(strFilepath);
+        read_handle.open(file_uri, Gnome::Vfs::OPEN_READ);
+
+        //It does (there was no exception), so ask the user to confirm overwrite:
+        bool bOverwrite = ui_ask_overwrite(file_uri);
 
         //Respond to button that was clicked:
-        bUseThisFilePath = bOverwrite;
+        bUseThisFileUri = bOverwrite;
+      }
+      catch(const Gnome::Vfs::exception& ex)
+      {
+        bUseThisFileUri = true; //It does not exist.
       }
     }
 
     //Save to this filepath:
-    if(bUseThisFilePath)
+    if(bUseThisFileUri)
     {
-      m_pDocument->set_filepath(strFilepath, true); //true = enforce file path
+      m_pDocument->set_file_uri(file_uri, true); //true = enforce file path
       bool bTest  = m_pDocument->save();
 
       if(!bTest)
@@ -270,7 +263,7 @@ void App_WithDoc::on_menu_file_save()
   if(m_pDocument)
   {
     //If there is already a filepath, then save to that location:
-    if(!(m_pDocument->get_filepath().empty()))
+    if(!(m_pDocument->get_file_uri().empty()))
     {
       bool bTest = m_pDocument->save();
 
@@ -306,7 +299,7 @@ void App_WithDoc::on_menu_file_save()
 }
 
 void App_WithDoc::init()
-{
+{  
   init_create_document();
 
   //Call base method:
@@ -315,9 +308,6 @@ void App_WithDoc::init()
   on_document_load(); //Show default empty document in the View.
 
   set_document_modified(false); //Disable Save menu item.
-
-  //Fill the Recent Documents sub menu:
-  session_load_document_history();
 }
 
 void App_WithDoc::init_create_document()
@@ -331,7 +321,7 @@ void App_WithDoc::init_create_document()
 
   m_pDocument->set_is_new(true); //Can't be modified if it's just been created.
   
-  m_pDocument->signal_modified().connect(SigC::slot(*this, &App_WithDoc::on_document_modified));
+  m_pDocument->signal_modified().connect(sigc::mem_fun(*this, &App_WithDoc::on_document_modified));
 
   update_window_title();
 }
@@ -425,7 +415,7 @@ void App_WithDoc::update_window_title()
 
 }
 
-void App_WithDoc::on_document_modified()
+void App_WithDoc::on_document_modified(bool modified)
 {
   //Change the displayed 'modified' status.
   //This method could be overridden to e.g. enable a Save icon or enable the Save menu item.
@@ -471,7 +461,7 @@ void App_WithDoc::after_successful_save()
   set_document_modified(false); //enables/disables menu and toolbar widgets.
     
   //Update document history list:
-  document_history_add(m_pDocument->get_filepath());     
+  document_history_add(m_pDocument->get_file_uri());     
 }
 
 Glib::ustring App_WithDoc::get_conf_fullkey(const Glib::ustring& key)
@@ -480,108 +470,15 @@ Glib::ustring App_WithDoc::get_conf_fullkey(const Glib::ustring& key)
 }
 
 
-void App_WithDoc::session_save_document_history()
+void App_WithDoc::document_history_add(const Glib::ustring& file_uri)
 {
-  //Save document history:
-
-  Glib::RefPtr<Gnome::Conf::Client> refConfClient = Gnome::Conf::Client::get_default_client();
-  
-  try
-  {
-    Glib::ustring key = get_conf_fullkey(BAKERY_GCONF_DIRECTORY_RECENTFILES);
-    refConfClient->set_string_list(key, m_vecDocHistory);
-  }
-  catch(const Gnome::Conf::Error& ex)
-  {
-    std::cerr << "Bakery: App_WithDoc::session_save_document_history(): " << ex.what() << std::endl;
-  }
+  //Override this.
 }
 
-void App_WithDoc::session_load_document_history()
-{
-  //Load document history:
-
-  Glib::RefPtr<Gnome::Conf::Client> refConfClient = Gnome::Conf::Client::get_default_client();
-
-  try
-  {
-    type_vecStrings vecTemp = refConfClient->get_string_list(get_conf_fullkey( BAKERY_GCONF_DIRECTORY_RECENTFILES ));
-
-    //Check that it has not become corrupted:
-    if(vecTemp.size() == m_vecDocHistory.size())
-      m_vecDocHistory = vecTemp;
-  }
-  catch(const Gnome::Conf::Error& ex)
-  {
-    //Maybe the directory doesn't exist yet. It's not a problem.
-    //std::cerr << "Bakery: App_WithDoc::session_save_document_history(): " << ex.what() << std::endl;
-  }
-
-  show_document_history_in_all_instances();
-}
-
-
-void App_WithDoc::document_history_add(const Glib::ustring& strFilepath)
-{
-  //TODO synchronize this static data.
-
-  //If it's not already there:
-  type_vecStrings::iterator iterFind = std::find(m_vecDocHistory.begin(), m_vecDocHistory.end(), strFilepath);
-  if(iterFind == m_vecDocHistory.end())
-  {
-    guint sizeHistory = m_vecDocHistory.size(); //Remember size so that we can keep it the same.
-    
-    //Add new item:
-    m_vecDocHistory.insert(m_vecDocHistory.begin(), strFilepath); //Inserts at the start, before begin().
- 
-    //Enforce size, possibly forgetting old history at the end:
-    m_vecDocHistory.resize(sizeHistory);
-   
-    session_save_document_history();
-
-    show_document_history_in_all_instances();
-  }
-}
-
-void App_WithDoc::document_history_remove(const Glib::ustring& strFilepath)
-{
-  //TODO synchronize this static data.
-
-  guint sizeHistory = m_vecDocHistory.size(); //Remember size so that we can keep it the same.
-
-  //Remove it if it's there:
-  type_vecStrings::iterator iterFind = std::find(m_vecDocHistory.begin(), m_vecDocHistory.end(), strFilepath);
-  if(iterFind != m_vecDocHistory.end())
-  {
-    m_vecDocHistory.erase(iterFind);
-
-    //Enforce size:
-    m_vecDocHistory.resize(sizeHistory);
-
-    session_save_document_history();
-
-    show_document_history_in_all_instances();
-  }
-}
-
-
-void App_WithDoc::show_document_history()
+void App_WithDoc::document_history_remove(const Glib::ustring& file_uri)
 {
   //Override this.
 }
 
 
-  
-void App_WithDoc::show_document_history_in_all_instances() //static
-{
-  //Update File menu in all instances:
-  AppInstanceManager::type_listAppInstances apps = m_AppInstanceManager.get_instances();
-  for(AppInstanceManager::type_listAppInstances::iterator iter = apps.begin(); iter != apps.end(); iter++)
-  {
-    App_WithDoc* pApp = dynamic_cast<App_WithDoc*>(*iter);
-    if(pApp)
-      pApp->show_document_history();
-  }
-}
-
 } //namespace
