@@ -8,7 +8,7 @@ namespace Bakery
 
 
 Dialog_OfferSave::Dialog_OfferSave(const Glib::ustring& filepath)
-: Gtk::MessageDialog(gettext("This document has unsaved changes. Would you like to save it?\n\nDocument:\n") + filepath, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE)
+: Gtk::MessageDialog(gettext("This document has unsaved changes. Would you like to save it?\n\nDocument:\n") + filepath, false /* no markup */, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE)
 {
   set_title(gettext("Save Changes?"));
 
