@@ -7,7 +7,7 @@ namespace Bakery
 
 
 Dialog_OverwriteFile::Dialog_OverwriteFile(const Glib::ustring& filepath)
-: Gtk::MessageDialog(gettext("Do you want to overwrite the existing file?\n\nDocument:\n") + filepath, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE)
+: Gtk::MessageDialog(gettext("Do you want to overwrite the existing file?\n\nDocument:\n") + filepath, false /* no markup */, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE)
 {
   set_title(gettext("Overwrite File?"));
 
